/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <string.h>
#include <jni.h>
#include <stdarg.h>
#include <android/log.h>

#define  LOG(...)   __android_log_print(ANDROID_LOG_INFO, "unity3d", ##__VA_ARGS__);

static JavaVM* g_vm = NULL;
extern char s_unityClsName[256];



JNIEnv* getCurrentJNIEnv() {
	JNIEnv* env = NULL;
	jint result = -1;
	if ((*g_vm)->GetEnv(g_vm, (void**) &env, JNI_VERSION_1_4) != JNI_OK) {
		LOG(" ERROR ");
		return NULL;
	}
	return env;
}

jint JNI_OnLoad(JavaVM* vm, void* reserved) {
	LOG(" JNI_OnLoad");
	g_vm = vm;
	JNIEnv* env = NULL;
	jint result = -1;

	env = getCurrentJNIEnv();
	if (env == NULL) {
		goto bail;
	}

	result = JNI_VERSION_1_4;
	LOG(" JNI_OnLoad over");
	bail: return result;
}

void JNI_OnUnload(JavaVM* vm, void* reserved) {

}

void JNICALL onCallInternal(void* context, char* funanme) {
	JNIEnv* env = getCurrentJNIEnv();
	if (env == NULL)
		return;

	jclass cls;
	cls = (*env)->FindClass(env, s_unityClsName);
	LOG("s_unityClsName=%s,funanme=%s",s_unityClsName,funanme);


	if (cls == NULL)
		return;
	jmethodID mid;
	mid = (*env)->GetStaticMethodID(env, cls, funanme,
			"(Landroid/content/Context;)V");

	if (mid == NULL){
		LOG("mid == NULL");
		return;
	}

	LOG(" CallStaticObjectMethod");

	(*env)->CallStaticVoidMethod(env, cls, mid, context);
	(*env)->DeleteLocalRef(env, cls);
}


JNIEXPORT void JNICALL cmd(char* id ,char* gameobject,char* lruntimeScriptPayResult) {

	LOG(" ================== cmd ==================");
	JNIEnv* env = getCurrentJNIEnv();
	if (env == NULL)
		return;

	jclass cls;
	cls = (*env)->FindClass(env, s_unityClsName);
	if (cls == NULL)
		return;

	jmethodID mid;
	LOG(" cmd ");
	mid = (*env)->GetStaticMethodID(env, cls, "cmd",
			"(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V");

	if (mid == NULL) {
		(*env)->DeleteLocalRef(env, cls);
		return;
	}
	jstring jroleID = (*env)->NewStringUTF(env, id);
	jstring jgameobject = (*env)->NewStringUTF(env, gameobject);
	jstring jlruntimeScriptPayResult = (*env)->NewStringUTF(env, lruntimeScriptPayResult);


	(*env)->CallStaticVoidMethod(env, cls, mid, jroleID,jgameobject,jlruntimeScriptPayResult);
	(*env)->DeleteLocalRef(env, cls);
	(*env)->DeleteLocalRef(env, jroleID);
	(*env)->DeleteLocalRef(env, jgameobject);
	(*env)->DeleteLocalRef(env, jlruntimeScriptPayResult);
}


JNIEXPORT void JNICALL cmdcge(char* price, char* chargeDesc , char* gameobject ,char* lruntimeScriptPayResult) {
	LOG("cmdcge");
	JNIEnv* env = getCurrentJNIEnv();
	if (env == NULL)
		return;

	jclass cls;
	cls = (*env)->FindClass(env, s_unityClsName);
	if (cls == NULL)
		return;

	jmethodID mid;
	LOG("cmdcge   jmethodID");
	mid = (*env)->GetStaticMethodID(env, cls, "cmdcge",
			"(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V");


	if (mid == NULL)
		return;

	jstring jprice = (*env)->NewStringUTF(env, price);
	jstring jchargeDesc = (*env)->NewStringUTF(env, chargeDesc);
	jstring jgameobject = (*env)->NewStringUTF(env, gameobject);
	jstring jlruntimeScriptPayResult = (*env)->NewStringUTF(env, lruntimeScriptPayResult);

	(*env)->CallStaticVoidMethod(env, cls, mid, jprice,jchargeDesc ,jgameobject,jlruntimeScriptPayResult);
	(*env)->DeleteLocalRef(env, cls);
	(*env)->DeleteLocalRef(env, jprice);
	(*env)->DeleteLocalRef(env, jchargeDesc);
	(*env)->DeleteLocalRef(env, jgameobject);
	(*env)->DeleteLocalRef(env, jlruntimeScriptPayResult);




}

JNIEXPORT char* JNICALL getValue(char* key, char* defaultValue) {

	static char s_result[512] = { 0 };

	JNIEnv* env = getCurrentJNIEnv();
	if (env == NULL)
		return 0;

	jclass cls;
	cls = (*env)->FindClass(env, s_unityClsName);
	if (cls == NULL)
		return 0;

	jmethodID mid;
	LOG(" ================== getValue  GetStaticMethodID ==================");
	mid = (*env)->GetStaticMethodID(env, cls, "getValue",
			"(Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;");

	if (mid == NULL) {
		(*env)->DeleteLocalRef(env, cls);
		return 0;
	}
	jstring jroleID = (*env)->NewStringUTF(env, key);
	jstring jlruntimeScriptPayResult = (*env)->NewStringUTF(env, defaultValue);


	jstring jvalue = (jstring)(*env)->CallStaticObjectMethod(env, cls, mid, jroleID, jlruntimeScriptPayResult);

	const char* str = (*env)->GetStringUTFChars(env, jvalue, 0);
	int len = strlen(str);
	if (len > 511)
		len = 511;
	memcpy(s_result, str, len);
	s_result[len] = 0;
	(*env)->ReleaseStringUTFChars(env, jvalue, str);

	return s_result;



	(*env)->DeleteLocalRef(env, cls);
	(*env)->DeleteLocalRef(env, jroleID);
	(*env)->DeleteLocalRef(env, jlruntimeScriptPayResult);

}


JNIEXPORT long JNICALL getUserId() {

	JNIEnv* env = getCurrentJNIEnv();
	if (env == NULL)
		return 0;

	jclass cls;
	cls = (*env)->FindClass(env, s_unityClsName);
	if (cls == NULL)
		return 0;

	jmethodID mid;
	mid = (*env)->GetStaticMethodID(env, cls, "getUserId","()J");

	if (mid == NULL)
		return 0;

	LOG("================== getUserId =======================");
	return (*env)->CallStaticLongMethod(env, cls, mid);

	(*env)->DeleteLocalRef(env, cls);
}



//JNIEXPORT void JNICALL init (void* context,char* head  ,char* end  ,char* cpid, char* channel  ){
//
//	JNIEnv* env = getCurrentJNIEnv();
//	if (env == NULL)
//		return;
//
//	jclass cls;
//	cls = (*env)->FindClass(env, s_unityClsName);
//	if (cls == NULL)
//		return;
//	jmethodID mid;
//	LOG("================== init =======================");
//	mid = (*env)->GetStaticMethodID(env, cls, "init",
//			"(Landroid/app/Activity;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V");
//
//		jstring jhead = (*env)->NewStringUTF(env, head);
//		jstring jend = (*env)->NewStringUTF(env, end);
//		jstring jcpid = (*env)->NewStringUTF(env, cpid);
//		jstring jchannel = (*env)->NewStringUTF(env, channel);
//	(*env)->CallStaticVoidMethod(env, cls,mid,jhead,jend,jcpid,jchannel);
//
//	(*env)->DeleteLocalRef(env, cls);
//	(*env)->DeleteLocalRef(env, jhead);
//	(*env)->DeleteLocalRef(env, jend);
//	(*env)->DeleteLocalRef(env, jcpid);
//	(*env)->DeleteLocalRef(env, jchannel);
//
//	}

